//
//	LogAs v 2.0
//	Russell Lowke, February 4th 2010
//	for use with ActionScript 3 (AS3, Flex SDK 3.2, Flash Player 9)
// 
//	Copyright (c) 2009-2010 Russell Lowke
//	see http://www.lowkemedia.com for more information
// 
//	Permission is hereby granted, free of charge, to any person obtaining a 
//	copy of this software and associated documentation files (the "Software"), 
//	to deal in the Software without restriction, including without limitation 
//	the rights to use, copy, modify, merge, publish, distribute, sublicense, 
//	and/or sell copies of the Software, and to permit persons to whom the 
//	Software is furnished to do so, subject to the following conditions:
// 
//	The above copyright notice and this permission notice shall be included in 
//	all copies or substantial portions of the Software.
// 
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
//	IN THE SOFTWARE. 
//
//

//
//	LogAs package
//
//	LogAs is useful in production environments where problems need to be logged
//	and dealt with rather than causing a crash. In particular, LogAs allows 
//	issues to be classed as either SEVERE, WARNING, INFO, or DEBUG. 
//
//
//	Usage:
//
//	LogAs can dispatch a classed log with an ID by using,
//
//		LogAs.log(LogAs.SEVERE, "Your log message", 10000);
//
//	Where the 1st parameter can be set to LogAs.SEVERE, LogAs.WARNING, 
//	LogAs.INFO, or LogAs.DEBUG, and the 3rd parameter allows for an
//	optional id value which defaults to 0.
//
//	There are also shorthand static helper methods, LogAs.severe(), 
//	LogAs.warning(), LogAs.info(), and LogAs.debug(), e.g.
//
//		LogAs.severe("Your error message");
//		LogAs.warning("Your warning message");
//		LogAs.info("Your info message");
//		LogAs.debug("Your debug message");
//	
//
//	You can set the logging level using LogAs.logLevel,
//	
//		LogAs.logLevel = LogAs.INFO;
//	
//	There are 5 logging levels, they are,
//
//		LogAs.logLevel = LogAs.NONE				// don't log
//		LogAs.logLevel = LogAs.SEVERE			// show only severe logs
//		LogAs.logLevel = LogAs.WARNING			// show severe and warning logs
//		LogAs.logLevel = LogAs.INFO				// show severe, warning and info logs
//		LogAs.logLevel = LogAs.DEBUG			// show all logs
//
//	By default the logLevel is set to INFO.
//
//	LogAs will automatically trace() logs. You can prevent LogAs  
//	from doing this by setting the traceLogs parameter to false,
//
//		LogAs.traceLogs = false;
//
//
//	You can also listen for logs using your own method,
//
//		import com.lowke.utils.logAs.events.LogAsEvent;
//
//		LogAs.addEventListener(LogAsEvent.LOG_AS_EVENT, handleLogs);
//
//
//	It's useful to use LogAs in conjuction with the Alert class.
//
//	Your handleLogs() method, if used in conjunction with the Alert class,
//	might look something like this,
//
//		import com.lowke.utils.alert.Alert;
//
//		public function handleLogs(evt:LogAsEvent):void {
//			Alert.defaultParent = stage;
//			var msg:String = evt.toString();
//			switch(evt.level) {
//				case LogAsEvent.SEVERE:
//					// show severe log as error alert
//					Alert.error(msg);
//					break;
//				case LogAsEvent.WARNING:
//					// show warning log as warning alert
//					Alert.warning(msg);
//					break;
//				case LogAsEvent.INFO:
//					// show info log as info alert
//					Alert.info(msg);
//					break;
//				case LogAsEvent.DEBUG:
//					// trace debug logs
//					trace(msg);
//					break;
//			}
//		}
//


package com.lowke.utils.logAs {
	
	import com.lowke.utils.logAs.events.LogAsEvent;
	
	import flash.events.Event;
	import flash.events.EventDispatcher;


	// dispatched when a log occurs.
	[Event(name="logAs_Event", type="com.lowke.utils.logAs.events.LogAsEvent")];


	public class LogAs extends EventDispatcher {
		
		// dispatched events
		public static const LOG_AS_EVENT:String = LogAsEvent.LOG_AS_EVENT;
		
		// log levels
		public static const NONE:uint	= 0;						// don't log
		public static const SEVERE:uint	 = LogAsEvent.SEVERE;		// show only severe logs
		public static const WARNING:uint = LogAsEvent.WARNING;		// show severe and warning logs
		public static const INFO:uint	 = LogAsEvent.INFO;			// show severe, warning and info logs
		public static const DEBUG:uint	 = LogAsEvent.DEBUG;		// show all logs		
		
		// static reference to LogAs singleton
		private static const INSTANCE:LogAs = new LogAs();
		
		private var _traceLogs:Boolean = true;						// if true logs are traced
		private var _logLevel:uint = INFO;							// log all except debug
		
		public function LogAs() {
			super();
			
			if (INSTANCE != null) {
				 // throw error to prevent new instances of LogAs being created.
				throw new Error("LogAs is a singleton, it should never be created twice.\r" + 
								"Use LogAs.instance to get a reference to LogAs.");
			}
		}
		
		public function register(obj:EventDispatcher):void {
			obj.addEventListener(LOG_AS_EVENT, handleLogAsEvent, false, 0, true);
		}
		public static function register(obj:EventDispatcher):void {
			INSTANCE.register(obj);
		}
		private function handleLogAsEvent(evt:LogAsEvent):void {
			
			if (evt.level <= _logLevel) {
				if (_traceLogs) {
					trace(evt);
				}
				
				// redispatch log event for others to listen to
				dispatchEvent(evt);
			}
		}
		
		
		//
		// Methods for directly triggering LogAs events
		//
		
		//
		// log() dispatches LogAsEvents, allowing an application
		// to deal with issues on its own terms, usually for logging purposes.
		public function log(level:uint, message:String, id:int = 0):void {
		
			// dispatch log event
			handleLogAsEvent(new LogAsEvent(LOG_AS_EVENT, message, level, id));
		}
		
		//
		// static helper methods
		
		public static function log(level:uint, message:String, id:int = 0):void {
			INSTANCE.log(level, message, id);
		}
		public static function severe(message:String, id:int = 0):void {
			INSTANCE.log(SEVERE, message, id);
		}
		public static function warning(message:String, id:int = 0):void {
			INSTANCE.log(WARNING, message, id);
		}
		public static function info(message:String, id:int = 0):void {
			INSTANCE.log(INFO, message, id);
		}
		public static function debug(message:String, id:int = 0):void {
			INSTANCE.log(DEBUG, message, id);
		}
		
		public static function addEventListener(type:String, listener:Function, useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = false):void {
			INSTANCE.addEventListener(type, listener, useCapture, priority, useWeakReference);
		}
		public static function removeEventListener(type:String, listener:Function, useCapture:Boolean = false):void {
			INSTANCE.removeEventListener(type, listener, useCapture);
		}
		
		//
		// accessors and mutators
		//

		public static function get instance():LogAs					{ return INSTANCE; }
		public static function get traceLogs():Boolean				{ return INSTANCE.traceLogs; }
		public static function set traceLogs(val:Boolean):void		{ INSTANCE.traceLogs = val; }
		public static function get logLevel():uint					{ return INSTANCE.logLevel; }
		public static function set logLevel(val:uint):void			{ INSTANCE.logLevel = val; }
		
		public function get traceLogs():Boolean						{ return _traceLogs; }
		public function set traceLogs(val:Boolean):void				{ _traceLogs = val; }
		public function get logLevel():uint							{ return _logLevel; }
		public function set logLevel(val:uint):void					{ _logLevel = val; }
	}
}